import csv
import logging
import os

logger = logging.getLogger(__name__)

LEGACY_FUELTECH_MAP = {
    "brown_coal": "coal_brown",
    "black_coal": "coal_black",
    "solar": "solar_utility",
    "biomass": "bioenergy_biomass",
}


DATA_PATH = os.path.join(os.path.dirname(__file__), "data")


def clean_fueltech(ft):
    """
        Clean the fueltech strings that come out of the spreadsheets
        or other sources

        @TODO replace with a compiled regexp
    """
    if not type(ft) is str:
        return None

    if ft == "-":
        return None

    ft = ft.lower().strip()

    if ft == "":
        return None

    return ft


def load_fueltech_map(fixture_name):
    """
        Reads the CSV to load the fueltech map

        Fields are:

        fuel_source,fuel_source_desc,tech,tech_desc,fueltech_map,load_type

    """

    MAP_KEYS = [
        "fuel_source",
        "fuel_source_desc",
        "tech",
        "tech_desc",
        "load_type",
    ]

    fixture_path = os.path.join(DATA_PATH, fixture_name)

    if not os.path.isfile(fixture_path):
        raise Exception("Not a file: {}".format(fixture_path))

    fueltech_map = {}

    # Funky encoding because of save from Excel .. leave it
    with open(fixture_path, encoding="utf-8-sig") as fh:
        csvreader = csv.DictReader(fh)
        for line in csvreader:
            record = [
                i or None for field, i in line.items() if field in MAP_KEYS
            ]

            key = tuple(map(clean_fueltech, record))

            fueltech_map[key] = line["fueltech_map"]

    return fueltech_map


FUELTECH_MAP = load_fueltech_map("aemo_fueltech_map.csv")


def lookup_fueltech(
    fueltype: str,
    fueltype_desc=None,
    techtype=None,
    techtype_desc=None,
    dispatch_type: str = "generator",
):
    """
        Takes fueltech strings from AEMO or other sources and maps them
        to opennem fueltechs using the csv file in the data directory

    """
    ft = clean_fueltech(fueltype)
    tt = clean_fueltech(techtype)
    ftd = clean_fueltech(fueltype_desc)
    ttd = clean_fueltech(techtype_desc)

    if not dispatch_type in ["generator", "load"]:
        raise Exception("Invalid dispatch type: {}".format(dispatch_type))

    lookup_set = tuple([ft, ftd, tt, ttd, dispatch_type])

    # Lookup legacy fuel tech types and map them
    if ft in LEGACY_FUELTECH_MAP.keys():
        return LEGACY_FUELTECH_MAP[ft]

    if lookup_set in FUELTECH_MAP:
        return FUELTECH_MAP[lookup_set]

    logger.warning(
        "Found fueltech {}, {}, {}, {} with no mapping".format(
            ft, tt, ftd, ttd
        )
    )

    return None



import json
import logging
from datetime import datetime
from itertools import groupby

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import MultipleResultsFound
from sqlalchemy.sql import text

from opennem.core.facilitystations import facility_station_join_by_name
from opennem.core.facilitystatus import lookup_facility_status
from opennem.core.fueltechs import lookup_fueltech
from opennem.core.normalizers import (
    clean_capacity,
    name_normalizer,
    normalize_duid,
    participant_name_filter,
    station_name_cleaner,
)
from opennem.core.unit_codes import get_unit_code
from opennem.core.unit_parser import parse_unit_duid
from opennem.db.models.opennem import (
    Facility,
    FacilityStatus,
    Participant,
    Station,
)
from opennem.pipelines import DatabaseStoreBase
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


def has_unique_duid(units: list) -> bool:
    duids = set([i["duid"] for i in units])
    return len(duids) == len(units)


def get_unique_duid(units: list) -> str:
    if not type(units) is list or len(units) < 1:
        return None

    first_record = units[0]

    return (
        normalize_duid(first_record["duid"])
        if "duid" in first_record
        else None
    )


def get_station_record_from_facilities(units: list):
    if not type(units) is list or len(units) < 1:
        raise Exception("Passed units list with no units ..")

    if len(units) == 1:
        return units[0]

    for u in units:
        cap = clean_capacity(u["reg_cap"])
        duid = normalize_duid(u["duid"])

        if cap:
            return u

    return units[0]


class RegistrationExemptionGrouperPipeline(object):
    """
        This is the first-pass pipeline of the AEMO Registration and Exemptions list

        It groups by clean station name

    """

    @check_spider_pipeline
    def process_item(self, item, spider=None):
        if not "generators" in item:
            raise Exception("No generators found in item pipeline failed")

        generators = item["generators"]

        # Add clean station names and if group_by name
        generators = [
            {
                **i,
                "name": station_name_cleaner(i["station_name"]),
                "name_join": False
                if facility_station_join_by_name(
                    station_name_cleaner(i["station_name"])
                )
                else i["duid"],
            }
            for i in generators
        ]

        # sort by name

        generators_grouped = {}

        for k, v in groupby(
            generators, key=lambda v: (v["name"], v["name_join"])
        ):
            key = k[0]
            if not key in generators_grouped:
                generators_grouped[key] = []

            generators_grouped[key] += list(v)

        return {**item, "generators": generators_grouped}


class RegistrationExemptionStorePipeline(DatabaseStoreBase):
    """
        This pipeline is the second pass of the AEMO NEM REL List

        It sorts out whats new, what is updated, units and updating the database

    """

    # Cache of participant keys
    participant_keys = {}

    def process_participants(self, participants):
        s = self.session()

        for participant_data in participants:
            participant_name = participant_data["name"].strip()

            try:
                participant = (
                    s.query(Participant)
                    .filter(Participant.name == participant_name)
                    .one_or_none()
                )
            except MultipleResultsFound as e:
                logger.info(
                    "Found multiple participants with name {}".format(
                        participant_name
                    )
                )

                participant = (
                    s.query(Participant)
                    .filter(Participant.name == participant_name)
                    .first()
                )

            if not participant:
                participant = Participant(name=participant_name)

            participant.name_clean = participant_name_filter(participant_name)
            participant.abn = participant_data["abn"]

            s.add(participant)

            logger.info(
                "Found new NEM participant: {}".format(participant_name)
            )
            self.participant_keys[participant_name] = participant

            facility = None

        s.commit()

    def process_facilities(self, generators):
        s = self.session()

        station = None
        participant = None
        station_current = None

        join_key = None

        stations_updated = 0
        stations_added = 0
        generators_updated = 0
        generators_added = 0

        for generator_data in generators:
            facility = None

            if not generator_data["dispatch_type"] == "Generator":
                continue

            station_name = name_normalizer(generator_data["station_name"])
            station_name_clean = station_name_cleaner(
                generator_data["station_name"]
            )
            participant_name = name_normalizer(generator_data["participant"])
            facility_region = name_normalizer(generator_data["region"])
            duid = normalize_duid(generator_data["duid"])
            reg_cap = clean_capacity(generator_data["reg_cap"])
            unit_no = (
                generator_data["unit_no"].strip()
                if type(generator_data["unit_no"]) is str
                else str(generator_data["unit_no"])
            )

            if join_key != station_name_clean:
                facility_station = None
                join_key = station_name_clean

            # @TODO insert into units table
            if not reg_cap:
                continue

            # See if we have the facility by duid
            facility = (
                s.query(Facility).filter(Facility.code == duid).one_or_none()
            )

            if not facility:
                logger.info(
                    "Skipping: Could not find facility {} ({})".format(
                        station_name, duid
                    )
                )
                continue

            facility_station = facility.station

            # Done trying to find existing

            created_station = False
            created_facility = False

            if not facility_station:
                facility_station = Station(
                    name=station_name,
                    name_clean=station_name_clean,
                    nem_region=facility_region,
                    created_by="pipeline.nemstorerel",
                )

                # s.add(facility_station)
                # s.commit()

                created_station = True

            if not facility:
                facility = Facility(
                    name=station_name,
                    code=duid,
                    name_clean=station_name_clean,
                    region=facility_region,
                    created_by="pipeline.nemstorerel",
                )
                facility.station = facility_station

                created_facility = True

            if duid:
                facility.code = duid

            if not facility.participant_id:
                facility.participant = facility_station.participant

            # Assume all REL's are operating if we don't have a status
            if not facility.status_id:
                facility.status_id = "operating"

            facility.unit_size = clean_capacity(generator_data["unit_size"])
            facility.unit_number = unit_no
            facility.region = facility_region
            facility.name = station_name
            facility.name_clean = station_name_clean
            facility.nameplate_capacity = reg_cap

            fueltech = lookup_fueltech(
                generator_data["fuel_source_primary"],
                generator_data["fuel_source_descriptor"],
                generator_data["tech_primary"],
                generator_data["tech_primary_descriptor"],
            )

            facility.fueltech_id = fueltech

            s.add(facility)
            s.commit()

            if created_station:
                logger.info(
                    "{} station with name {} and id {}".format(
                        "Created" if created_station else "Updated",
                        facility_station.name_clean,
                        facility_station.id,
                    )
                )

            if created_facility:
                logger.info(
                    "{} facility with name {} and duid {} and id {}".format(
                        "Created" if created_facility else "Updated",
                        facility.name_clean,
                        facility.code,
                        facility.id,
                    )
                )

            generators_updated += 1

        print(
            "NEM REL Pipeline: Added {} stations, updated {} stations. Added {}, updated {} generators of {} total".format(
                stations_added,
                stations_updated,
                generators_added,
                generators_updated,
                len(generators),
            )
        )

    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        if not "generators" in item and type(item["generators"]) is not list:
            raise Exception("Invalid item - no generators located")

        if (
            not "participants" in item
            and type(item["participants"]) is not list
        ):
            raise Exception("Invalid item - no generators located")

        generators = item["generators"]
        participants = item["participants"]

        if not type(generators) is dict:
            raise Exception(
                "Require grouped generators for this pipeline step. Run RegistrationExemptionGrouperPipeline"
            )

        self.process_generators(generators)

    def process_generators(self, generators):
        s = self.session()

        stations_updated = 0
        stations_added = 0
        generators_updated = 0
        generators_added = 0

        from pprint import pprint

        for station_name, facilities in generators.items():
            facility = None
            facility_station = None
            created_station = False
            created_facility = False

            duid_unique = has_unique_duid(facilities)
            facility_count = len(facilities)

            logger.debug(station_name)
            logger.debug(facilities)

            # Step 1. Find the station
            # First by duid if it's unique
            duid = get_unique_duid(facilities)

            # This is the most suitable unit record to use for the station
            # see helper above
            facility_station_record = get_station_record_from_facilities(
                facilities
            )

            if duid and duid_unique and facility_count == 1:

                facility_lookup = (
                    s.query(Facility)
                    .filter(Facility.network_code == duid)
                    .one_or_none()
                )

                if facility_lookup and facility_lookup.station:
                    facility_station = facility_lookup.station

            if (
                duid
                and (duid_unique and facility_count > 1)
                or not duid_unique
            ):

                facility_lookup = (
                    s.query(Facility)
                    .filter(Facility.network_code == duid)
                    .first()
                )

                if facility_lookup and facility_lookup.station:
                    facility_station = facility_lookup.station

            # Create one as it doesm't exist
            if not facility_station:
                facility_station = Station(
                    name=station_name,
                    network_name=name_normalizer(
                        facility_station_record["station_name"]
                    ),
                    network_id="NEM",
                    created_by="pipeline.aemo.registration_exemption",
                )

                s.add(facility_station)
                created_station = True

            # Step 2. Add the facilities/units to the station
            # Now that we have a station or created one ..

            # Step go through the facility records we got ..
            for facility_record in facilities:
                network_name = name_normalizer(facility_record["station_name"])
                participant_name = name_normalizer(
                    facility_record["participant"]
                )
                facility_region = name_normalizer(facility_record["region"])
                duid = normalize_duid(facility_record["duid"])
                reg_cap = clean_capacity(facility_record["reg_cap"])
                unit = parse_unit_duid(facility_record["unit_no"], duid)
                unit_size = clean_capacity(facility_record["unit_size"])
                unit_code = get_unit_code(
                    unit, duid, facility_station_record["station_name"]
                )
                facility_status = "operating"
                fueltech = lookup_fueltech(
                    facility_record["fuel_source_primary"],
                    facility_record["fuel_source_descriptor"],
                    facility_record["tech_primary"],
                    facility_record["tech_primary_descriptor"],
                )

                # check if we have it by ocode first
                facility = (
                    s.query(Facility)
                    .filter(Facility.code == unit_code)
                    .one_or_none()
                )

                # If the duid is unique then we have no issues on which to join/create
                if duid and duid_unique and not facility:
                    facility = (
                        s.query(Facility)
                        .filter(Facility.network_code == duid)
                        .one_or_none()
                    )

                # If the duid is not unique then we need to figure things out ..
                if duid and not duid_unique and not facility:
                    facility_lookup = (
                        s.query(Facility)
                        .filter(Facility.network_code == duid)
                        # Not having a code means we haven't written to this record yet so we'll use it
                        .filter(Facility.code == None)
                        .all()
                    )

                    facility_db_count = len(facility_lookup)

                    logging.debug(
                        "Non unique duid: {} with {} in database and {} in facility duid is {}".format(
                            station_name,
                            facility_db_count,
                            facility_count,
                            duid,
                        )
                    )

                    if len(facility_lookup) > 0:
                        facility = facility_lookup.pop()

                if not facility:
                    facility = Facility(
                        network_code=duid,
                        created_by="pipeline.aemo.registration_exemption",
                    )
                    created_facility = True

                # Sanity checking
                if len(unit_code) < 3:
                    raise Exception(
                        "Unit code {} is invalid. For station {} with duid {}".format(
                            unit_code, station_name, duid
                        )
                    )

                #
                facility.code = unit_code
                facility.fueltech_id = fueltech
                facility.network_code = duid
                facility.network_region = facility_region
                facility.network_name = network_name

                facility.capacity_registered = reg_cap

                facility.unit_id = unit.id
                facility.unit_number = unit.number
                facility.unit_size = unit_size
                facility.unit_alias = unit.alias
                facility.unit_capacity = unit_size

                # Assume all REL's are operating if we don't have a status
                facility.status_id = "operating"

                facility.station = facility_station

                # Log that we have a new fueltech
                if fueltech and fueltech != facility.fueltech_id:
                    logger.warn(
                        "Fueltech mismatch for {} {}: prev {} new {}".format(
                            facility.name_clean,
                            facility.code,
                            facility.fueltech_id,
                            fueltech,
                        )
                    )

                if not created_facility:
                    facility.updated_by = (
                        "pipeline.aemo.registration_exemption"
                    )

                s.add(facility)
                s.commit()

            if created_station:
                logger.info(
                    "{} station with name {} and id {}".format(
                        "Created" if created_station else "Updated",
                        facility_station.name,
                        facility_station.id,
                    )
                )

            if created_facility:
                logger.info(
                    "{} facility with name {} and duid {} and id {}".format(
                        "Created" if created_facility else "Updated",
                        facility.network_name,
                        facility.code,
                        facility.network_code,
                    )
                )

            generators_updated += 1

        print(
            "NEM REL Pipeline: Added {} stations, updated {} stations. Added {}, updated {} generators of {} total".format(
                stations_added,
                stations_updated,
                generators_added,
                generators_updated,
                len(generators),
            )
        )

        return 0



import json

from opennem.api.stations import get_stations
from opennem.exporter.encoders import OpenNEMJSONEncoder


def stations_json_records():
    stations = get_stations()

    records = []

    for station in stations:
        for facility in station.facilities:
            rec = {
                "name": station.name,
                # "oid": station.oid,
                "ocode": station.ocode,
                "code": facility.duid,
                "region": facility.network_region,
                "status": facility.status_id,
                "fueltech": facility.fueltech_id,
                "unit_id": facility.unit_id,
                "unit_num": facility.unit_number,
                "unit_cap": facility.capacity_aggregate,
                "station_cap_agg": station.capacity_aggregate,
                "station_cap_registered": station.capacity_registered,
                "added_by": facility.created_by,
                "updated_by": facility.created_by,
            }
            records.append(rec)

    return records


def stations_json_serialize():

    json_records = stations_json_records()

    json_serialized = json.dumps(
        json_records, cls=OpenNEMJSONEncoder, indent=4
    )

    return json_serialized



import json
import logging
from datetime import datetime
from itertools import groupby

from scrapy.exceptions import DropItem
from sqlalchemy.exc import IntegrityError
from sqlalchemy.orm.exc import MultipleResultsFound
from sqlalchemy.sql import text

from opennem.core.facilitystations import facility_station_join_by_name
from opennem.core.facilitystatus import lookup_facility_status
from opennem.core.fueltechs import lookup_fueltech
from opennem.core.normalizers import (
    clean_capacity,
    name_normalizer,
    normalize_duid,
    participant_name_filter,
    station_name_cleaner,
)
from opennem.db.models.opennem import (
    Facility,
    FacilityStatus,
    Participant,
    Station,
)
from opennem.pipelines import DatabaseStoreBase
from opennem.utils.pipelines import check_spider_pipeline

logger = logging.getLogger(__name__)


class GeneralInformationGrouperPipeline(object):
    """
        This is the first-pass pipeline of the AEMO general information list

        It groups by clean station name

    """

    @check_spider_pipeline
    def process_item(self, item, spider=None):
        if not "records" in item:
            raise Exception("No records found in item pipeline failed")

        generators = item["records"]

        # Add clean station names and if group_by name
        generators = [
            {
                **i,
                "name": station_name_cleaner(i["station_name"]),
                "name_join": False
                if facility_station_join_by_name(
                    station_name_cleaner(i["station_name"])
                )
                else i["duid"],
            }
            for i in generators
        ]

        # sort by name

        generators_grouped = {}

        for k, v in groupby(
            generators, key=lambda v: (v["name"], v["name_join"])
        ):
            key = k[0]
            if not key in generators_grouped:
                generators_grouped[key] = []

            generators_grouped[key] += list(v)

        return {**item, "generators": generators_grouped}


class GeneralInformationStoragePipeline(DatabaseStoreBase):
    def parse_interval(self, date_str):
        return datetime.strptime(date_str, "%Y-%m-%d %H:%M:%S")

    def process_participants(self, participants):
        for participant in participants:
            pass

    @check_spider_pipeline
    def process_item(self, item, spider=None):

        s = self.session()

        if item["FuelType"] in ["Natural Gas Pipeline"]:
            return {}

        if (
            item["UnitStatus"]
            in [
                "Publically Announced",
                "Upgrade",
                "Emerging",
                "Expansion",
                "Maturing",
            ]
            or not item["UnitStatus"]
        ):
            return {}

        participant_name = name_normalizer(item["Owner"])

        # Funky case of Todae solar where they put their name in the

        participant = (
            s.query(Participant)
            .filter(Participant.name == participant_name)
            .one_or_none()
        )

        if not participant:
            participant = Participant(
                name=participant_name,
                name_clean=participant_name_filter(participant_name),
            )

            s.add(participant)
            s.commit()
            logger.info(
                "GI: Added new partipant to NEM database: {}".format(
                    participant_name
                )
            )

        facility = None
        facility_station = None

        facility_name = name_normalizer(item["Name"])
        facility_name_clean = station_name_cleaner(item["Name"])
        duid = normalize_duid(item["DUID"])
        facility_status = lookup_facility_status(item["UnitStatus"])
        facility_region = item["Region"]
        facility_fueltech = (
            lookup_fueltech(item["FuelType"], techtype=item["TechType"])
            if ("FuelType" in item and item["FuelType"])
            else None
        )

        if facility_station_join_by_name(facility_name_clean):
            facility_station = (
                s.query(Station)
                .filter(Station.name_clean == facility_name_clean)
                .first()
            )

        if duid:
            try:
                facility = (
                    s.query(Facility)
                    .filter(Facility.code == duid)
                    .filter(Facility.region == facility_region)
                    # .filter(Facility.nameplate_capacity != None)
                    .one_or_none()
                )
            except MultipleResultsFound:
                logger.warn(
                    "Multiple results found for duid : {}".format(duid)
                )

            if facility:
                if not facility.station:
                    raise Exception(
                        "GI: Existing facility {} {} with no station .. unpossible.".format(
                            facility.id, facility.name
                        )
                    )

                if facility.station and not facility_station:
                    facility_station = facility.station

                logger.info(
                    "GI: Found facility by DUID: {} {} {}".format(
                        facility.id, facility.name, facility_station.id
                    )
                )

        if not duid:
            facility_lookup = (
                s.query(Facility)
                .filter(Facility.region == facility_region)
                .filter(Facility.name_clean == facility_name_clean)
                # .filter(Facility.fueltech_id == facility_fueltech)
                .filter(Facility.code == None)
                .first()
            )

            if facility_lookup:
                if not facility_lookup.station:
                    raise Exception(
                        "GI: Existing facility {} {} with no station .. unpossible.".format(
                            facility.id, facility.name
                        )
                    )

                if facility_lookup.station and not facility_station:
                    facility_station = facility_lookup.station

                # logger.info(
                #     "GI: Found facility by no duid name fueltech and region : {} {} {}".format(
                #         facility.id, facility.name, facility_station.id
                #     )
                # )

        # Done trying to find existing

        created_station = False
        created_facility = False

        if not facility_station:
            facility_station = Station(
                name=item["Name"],
                name_clean=facility_name_clean,
                nem_region=facility_region,
                created_by="pipeline.nem.facility.gi",
            )

            s.add(facility_station)

            created_station = True

        if not facility:
            facility = Facility(created_by="pipeline.nem.facility.gi")
            facility.station = facility_station

            created_facility = True

        if duid:
            facility.code = duid

        facility.participant = participant

        if not facility.region:
            facility.region = facility_region

        if not facility.name:
            facility.name = name_normalizer(item["Name"])

        if not facility.name_clean:
            facility.name_clean = station_name_cleaner(item["Name"])

        # @TODO parse units into ints
        facility.unit_number = item["Units"]

        if not facility.fueltech_id and facility_fueltech:
            facility.fueltech_id = facility_fueltech

        if facility.fueltech_id is None:
            logger.error("Could not find fueltech for: {}".format(item))

        if not facility.nameplate_capacity:
            facility.nameplate_capacity = clean_capacity(
                item["UpperCapacity"] or item["NameCapacity"]
            )

        facility.status_id = facility_status

        if facility_station and not facility.station:
            facility.station = facility_station

        if facility.status_id is None:
            raise Exception(
                "GI: Failed to map status ({}) on row: {}".format(
                    facility.status_id, item
                )
            )

        if created_station:
            logger.info(
                "GI: {} station with name {} and id {}".format(
                    "Created" if created_station else "Updated",
                    facility_station.name_clean,
                    facility_station.id,
                )
            )

        if created_facility:
            logger.info(
                "GI: {} facility with name {} and duid {} and id {}".format(
                    "Created" if created_facility else "Updated",
                    facility.name_clean,
                    facility.code,
                    facility.id,
                )
            )

        try:
            s.add(facility)
            s.commit()
        except Exception as e:
            logger.error(e)
            raise e
        finally:
            s.close()

        return {
            "name": name_normalizer(item["Name"]),
            "status": item["UnitStatus"],
            "status_id": lookup_facility_status(item["UnitStatus"]),
        }



# GULLRWF2 -> Biala
import json
from datetime import date, datetime
from pprint import pprint

from sqlalchemy.orm import sessionmaker

from opennem.db import db_connect
from opennem.db.models.opennem import metadata

engine = db_connect()
session = sessionmaker(bind=engine)


def patches():

    sqls = [
        "update facility set capacity_registered = 2.0, unit_capacity = 2.0 where code = 'GOSNELLS'",
        "update facility set capacity_registered = 1.1, unit_capacity = 1.1  where code = 'ATLAS'",
        # code GULLRWF2_74 -> Biala
        # "update facility set station_id = null where code = 'GULLRWF2'",
        # "update facility set code = 'GULLRWF2' where name_clean = 'Biala'",
        "update facility set station_id = (select id from station where name = 'Wivenhoe Small Hydro') where code ='WIVENSH'",
        "update station set name_clean = 'Wivenhoe Mini' where name_clean = 'Wivenhoe Small'",
        "update facility set fueltech_id = 'pumps' where code in ('PUMP2', 'PUMP1')",
        "update facility set station_id = null where name='Crookwell' and code is null",
        "update facility set station_id = null where name='Pioneer Sugar Mill' and code is null",
    ]

    with engine.connect() as c:

        for query in sqls:
            rows = c.execute(query)
            pprint(rows)


if __name__ == "__main__":
    patches()



import decimal
import json
import logging
from datetime import date, datetime
from decimal import Decimal
from pprint import pprint
from typing import List

from geojson import Feature, FeatureCollection, Point, dumps
from smart_open import open
from sqlalchemy.orm import sessionmaker

from opennem.db import db_connect
from opennem.db.models.opennem import (
    Facility,
    FacilityStatus,
    FuelTech,
    Network,
    Station,
    metadata,
)

engine = db_connect()
session = sessionmaker(bind=engine)

logger = logging.getLogger(__name__)


def get_stations() -> List[Station]:
    s = session()

    stations = (
        s.query(Station)
        .join(Facility)
        .join(FuelTech)
        # , Network, FacilityStatus)
        .filter(Facility.fueltech != None)
        .filter(Facility.status != None)
        .order_by(Station.network_region, Station.id, Facility.code)
        .all()
    )

    logger.info("Got {} stations".format(len(stations)))

    return stations


if __name__ == "__main__":
    get_stations()



from typing import Optional

from pydantic import BaseModel, ValidationError, validator

from opennem.core.normalizers import (
    clean_capacity,
    is_number,
    normalize_duid,
    normalize_string,
    participant_name_filter,
    station_name_cleaner,
)


class Participant(BaseModel):
    # id: int = Optional[int]
    code: Optional[str]
    name: str
    network_name: str
    country: str = "au"
    abn: Optional[str]

    @validator("code")
    def code_clean(cls, v):
        return normalize_duid(v)

    @validator("network_name")
    def network_name_isalpha(cls, v):
        assert type(v) is str, "must be string"
        # assert v.isalpha(), "must be alpha: {}".format(v)

        return v

    @validator("name")
    def name_clean(cls, v):
        name = participant_name_filter(v)

        return name

    class Config:
        orm_mode = True
        anystr_strip_whitespace = True
        # validate_assignment = True


class Station(BaseModel):
    id: int


class Station(BaseModel):
    id: int



"""
    Module that generated opennem ids

"""

from opennem.core import base24

# from opennem.db.models.opennem import Station


def get_oid(model) -> str:
    _id = model.id

    if not model.id:
        return None

    parent_oid = None

    if hasattr(model, "station"):
        parent_oid = get_oid(model.station)

    item_oid = base24.dumps(_id)

    oid_components = [
        parent_oid,
        item_oid,
    ]

    oid_string = "_".join([str(i) for i in oid_components if i])

    return oid_string


def get_ocode(station) -> str:
    """

        format: {country}_{network}_{region}_{station}_{id}
    """
    values = [
        station.network.country or "au",
        station.network.code or None,
        station.network_region or None,
        station.state or None,
        station.code or None,
        station.id,
    ]

    values = [str(i).lower() for i in values if i is not None]

    ocode = "_".join(values)

    return ocode
